#version 430

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedo;
uniform sampler2D gMetallicRoughness; // r-channel: metallic, g-channel: roughness, b-channel: currently unused

#ifdef SSAO
uniform sampler2D ssaoTexture;
#endif

out vec4 fragColor;

void main() {
    #ifdef NUM_LIGHTS
        vec3 position = texture(gPosition, vertexOutput.uv).rgb;
        vec3 normal = texture(gNormal, vertexOutput.uv).rgb;
        vec3 albedo = texture(gAlbedo, vertexOutput.uv).rgb;
        vec3 metallicRoughness = texture(gMetallicRoughness, vertexOutput.uv).rgb;

        vec3 viewDir = normalize(camera.position - position);

        Surface surf;
        surf.albedo = albedo;
        surf.metallic = metallicRoughness.r;
        surf.roughness = metallicRoughness.g;
        surf.normal = normal;
        surf.position = position;
        surf.view = viewDir;

        #ifdef SSAO
            fragColor = vec4(shade(surf, texture(ssaoTexture, vertexOutput.uv).r), 1);
        #else
            fragColor = vec4(shade(surf), 1);
        #endif
    #else
        discard;
    #endif
}
