#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedo;
uniform sampler2D gMetallicRoughness; // r-channel: metallic, g-channel: roughness, b-channel: currently unused

out vec4 fragColor;

const float coveredSolidAngle = (2 * pi) / NUM_SAMPLES;

/** Calculates the directional occlusion for one pixel.
*/
vec3 directionalOcclusion(vec3 position, vec3 viewSpacePosition, mat3 TBN, vec3 normal, samplerCube environment, float lod) {
    vec3 result = vec3(0);

    for (int i = 0; i < NUM_SAMPLES; ++i) {
        vec3 sampleDirection = TBN * samples[i].xyz;
        vec3 samplePoint = position + sampleDirection * samples[i].w;

        vec4 viewSpaceSample = camera.view * vec4(samplePoint, 1);
        vec2 sampleUV = getSampleUV(viewSpaceSample);
        vec3 viewSpaceOccluder = (camera.view * vec4(texture(gPosition, sampleUV).rgb, 1)).xyz;
        float distance = length(viewSpacePosition - viewSpaceOccluder);
        float visibility = getVisibility(viewSpaceSample.xyz, viewSpaceOccluder, distance);

        float cosTheta = dot(normal, sampleDirection);
        vec3 li = textureLod(environment, sampleDirection, lod).xyz;

        result += li * visibility * cosTheta;
        //result += abs(texture(gNormal, sampleUV).rgb);
        //result += vec3(1 - visibility);
        //result += vec3(distance);
        //result += sampleDirection;
    }
    return result * oneOverPi * coveredSolidAngle;
}

void main() {
	vec2 uv = vertexOutput.uv;

	vec3 position = texture(gPosition, uv).rgb;
	vec3 normal = texture(gNormal, uv).rgb;
    vec3 albedo = texture(gAlbedo, uv).rgb;
    vec3 metallicRoughness = texture(gMetallicRoughness, uv).rgb;
	float metallic = metallicRoughness.r;
	float roughness = metallicRoughness.g;

    vec3 viewSpacePosition = (camera.view * vec4(position, 1)).xyz;
    mat3 TBN = getCoordinateSystem(normal);
    TBN = getRotatedCoordinateSystem(normal, uv);

	vec3 viewDir = normalize(camera.position - position);
    float NdotV = max(dot(normal, viewDir), 0.0f);

	vec3 F0 = vec3(0.04); // we assume F0 = 0.04 for dielectrics
	F0 = mix(F0, albedo, metallic);	// metallic surfaces have a tinted reflection and no diffuse light - use albedo color here
	vec3 fresnel = fresnelSchlickRoughness(NdotV, F0, roughness);

	float mipLevel = mipLevelFromRoughness(roughness);
	float ssdoMipLevel = 1024;
	ssdoMipLevel = mipLevel;
	ssdoMipLevel = 0;

	vec3 kD = vec3(1.0) - fresnel;
	kD *= 1.0 - metallic; // no diffuse for metallics

	vec3 diffuse = vec3(0);

    diffuse += directionalOcclusion(position, viewSpacePosition, TBN, normal, skybox, ssdoMipLevel);

	diffuse = diffuse * kD * albedo;

	fragColor = vec4(diffuse, 1);
}

