#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedo;
uniform sampler2D gMetallicRoughness; // r-channel: metallic, g-channel: roughness, b-channel: currently unused

uniform sampler2D lightedScene;

out vec4 fragColor;

#ifdef SSDO
    float area = 50;

    float senderArea = (pi * (rMax * rMax)) / (NUM_SAMPLES * area);


    vec3 indirectLighting(vec3 position, vec3 viewSpacePosition, mat3 TBN, vec3 normal) {
        vec3 result = vec3(0);

        for (int i = 0; i < NUM_SAMPLES; ++i) {
            vec3 sampleDirection = TBN * samples[i].xyz;
            vec3 samplePoint = position + sampleDirection * samples[i].w;

            vec4 viewSpaceSample = camera.view * vec4(samplePoint, 1);

            vec2 sampleUV = getSampleUV(viewSpaceSample);
            vec3 worldSpaceOccluder = texture(gPosition, sampleUV).rgb;

            vec3 transmittanceDirection = worldSpaceOccluder - position;
            vec3 transmittanceDirectionNormalized = normalize(transmittanceDirection);

            float distance = min(1.0, length(transmittanceDirection));
            float distanceSquared = distance * distance;

            vec3 viewSpaceOccluder = (camera.view * vec4(worldSpaceOccluder, 1)).xyz;
            float visibility = 1.0 - getVisibility(viewSpaceSample.xyz, viewSpaceOccluder, distance);

            vec3 occluderNormal = texture(gNormal, sampleUV).rgb;

            float cosThetaSender = max(0.0, dot(occluderNormal, -transmittanceDirectionNormalized));
            float cosThetaReceiver = max(0.0, dot(normal, transmittanceDirectionNormalized));

            vec3 li = texture(lightedScene, sampleUV).rgb;

            result += max(vec3(0), li * visibility * ((cosThetaSender * cosThetaReceiver) / distanceSquared));
        }
        return result * oneOverPi * senderArea;
    }
#endif

void main() {
    vec2 uv = vertexOutput.uv;
    vec3 color = texture(lightedScene, uv).rgb;

    #ifdef SSDO
	    vec3 position = texture(gPosition, uv).rgb;
        vec3 normal = texture(gNormal, uv).rgb;
        vec3 albedo = texture(gAlbedo, uv).rgb;
        vec3 metallicRoughness = texture(gMetallicRoughness, uv).rgb;

        float metallic = metallicRoughness.r;
        float roughness = metallicRoughness.g;

        vec3 viewSpacePosition = (camera.view * vec4(position, 1)).xyz;
        mat3 TBN = getCoordinateSystem(normal);

    	vec3 viewDir = normalize(camera.position - position);
        float NdotV = max(dot(normal, viewDir), 0.0f);
        vec3 F0 = vec3(0.04);	// we assume F0 = 0.04 for dielectrics
        F0 = mix(F0, albedo, metallic);		// metallic surfaces have a tinted reflection and no diffuse light - use albedo color here
        vec3 fresnel = fresnelSchlickRoughness(NdotV, F0, roughness);
        vec3 kD = vec3(1.0) - fresnel;
        kD *= 1.0 - metallic;	// no diffuse for metallics

        vec3 diffuse = indirectLighting(position, viewSpacePosition, TBN, normal);

        diffuse = diffuse * kD * albedo;

        fragColor = vec4(diffuse, 1);
    #else
        fragColor = vec4(0, 0, 0, 1);
	#endif
}

