#version 430 core

#define MAX_MIP_LEVEL 512

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D lightedScene;
uniform sampler2D luminance;

out vec4 fragColor;


float log10( in float n ) 
{
	const float kLogBase10 = 1.0 / log2( 10.0 );
	return log2(n) * kLogBase10;
}

float luminanceFromColor(vec3 color)
{
	return dot(color, vec3(0.2126, 0.7152, 0.0722));
}

vec3 uncharted2Tonemap(vec3 x)
{
    float A = 0.15;
	float B = 0.50;
	float C = 0.10;
	float D = 0.20;
	float E = 0.02;
	float F = 0.30;

    return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

vec3 reinhard3(vec3 color, float avgLum, float key)
{
	float pixelLum = luminanceFromColor(color);

	float lum = (key / avgLum) * pixelLum;
	lum = lum / (1.0 + lum);

	return color * (lum / pixelLum);
}

vec3 reinhard4(vec3 color, float avgLum, float key, float whiteLevel)
{
	float pixelLum = luminanceFromColor(color);

	float lum = (key / avgLum) * pixelLum;
	lum = pixelLum * (1.0f + pixelLum / (whiteLevel * whiteLevel)) / (1.0f + pixelLum);

	return color * (lum / pixelLum);
}


void main() 
{
	vec3 color = texture(lightedScene, vertexOutput.uv).rgb;
	
#ifdef RENDER_LUMINANCE
	
	float pixelLum = luminanceFromColor(color);
	fragColor = vec4(min(log(0.0001 + pixelLum), 10.0), 0, 0, 0);

#else
	
	float avgLum = exp(textureLod(luminance, vec2(0.5), MAX_MIP_LEVEL).r);

	//color = color / (1.0 + color);	// Simplest

	// TODO check auto key calculation (doesn't work yet)
	// Taken from https://mynameismjp.wordpress.com/2010/04/30/a-closer-look-at-tone-mapping/
	//float key = 1.03f - (2.0f / (2 + log10(avgLum + 1)));
	//color = reinhard3(color, avgLum, 0.4);

	color = reinhard4(color, avgLum, 0.18, 2.0);
	
	//color = uncharted2Tonemap(color);
	
	fragColor = vec4(color, 1.0);
	
#endif
	
}
