#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
	vec2 surrounding[4];
} vertexOutput;

uniform sampler2D inputTexture;
uniform sampler2D lightingResult;
uniform sampler2D gPosition;

out vec4 fragColor;

void main() {
    vec2 uv = vertexOutput.uv;
    vec2 texelSize = 1.0 / textureSize(inputTexture, 0);

    float centerDepth = (camera.view * vec4(texture(gPosition, uv).rgb, 1.0)).z;

    float samples = 4;
    vec3 result = vec3(0.0);
    for (int i = 0; i < 4; ++i) {
        vec2 sampleUV = vertexOutput.surrounding[i];
        float localDepth = (camera.view * vec4(texture(gPosition, sampleUV).rgb, 1.0)).z;
        if (abs(centerDepth - localDepth) > 0.2) {
            samples--;
        } else {
            result += texture(inputTexture, sampleUV).rgb;
        }
    }
    result /= samples;
    vec3 lightingColor = texture(lightingResult, uv).rgb;

    vec3 combined = result + lightingColor;
    fragColor = vec4(combined, 1);
}
