#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedo;

//uniform sampler2D lightingResult;
uniform sampler2D diffuseBrdf;

out vec4 fragColor;

float vplIntensity = 0.3;
float area = 1;
float senderArea = (pi * (rMax * rMax)) / (NUM_SAMPLES * area);
const float coveredSolidAngle = (2 * pi) / NUM_SAMPLES;

/** Calculates the directional occlusion for one pixel.
*/
vec3 directionalOcclusion(vec3 position, vec3 viewSpacePosition, mat3 TBN, vec3 normal, samplerCube environment, float lod) {
    vec3 ssdoResult = vec3(0);
    vec3 indirectBounceResult = vec3(0);

    float v = 0.0;

    int usedSampes = NUM_SAMPLES;
    for (int i = 0; i < NUM_SAMPLES; ++i) {
        vec3 sampleDirection = TBN * samples[i].xyz;
        vec3 samplePoint = position + sampleDirection * samples[i].w;

        float cosThetaSample = dot(normal, sampleDirection);
        if (cosThetaSample > 0.2) {
            vec4 viewSpaceSample = camera.view * vec4(samplePoint, 1);

            vec2 sampleUV = getSampleUV(viewSpaceSample);
            vec3 worldSpaceOccluder = texture(gPosition, sampleUV).rgb;

            vec3 transmittanceDirection = worldSpaceOccluder - position;
            vec3 transmittanceDirectionNormalized = normalize(transmittanceDirection);

            float distance = min(1.0, length(transmittanceDirection));
            float distanceSquared = distance * distance;

            vec3 viewSpaceOccluder = (camera.view * vec4(worldSpaceOccluder, 1)).xyz;
            float visibility = getVisibility(viewSpaceSample.xyz, viewSpaceOccluder, distance);

            //vec3 occluderNormal = texture(gNormal, sampleUV).rgb;
            //float cosThetaSender = max(0.0, dot(occluderNormal, -transmittanceDirectionNormalized));
            //float cosThetaReceiver = max(0.0, dot(normal, transmittanceDirectionNormalized));
            //vec3 liOccluded = texture(lightingResult//gAlbedo
            //    , sampleUV).rgb * vplIntensity;
            //indirectBounceResult += max(vec3(0), liOccluded * (1 -visibility) * ((cosThetaSender * cosThetaReceiver) / distanceSquared));

            vec3 liVisible = textureLod(environment, sampleDirection, lod).xyz;

            ssdoResult += liVisible * visibility * cosThetaSample;
            v += visibility;
        } else {
            usedSampes--;
        }
    }
    //return vec3(v) / usedSampes;
    //return ((ssdoResult * coveredSolidAngle) + (indirectBounceResult * senderArea)) * oneOverPi;
    return (ssdoResult * coveredSolidAngle) * oneOverPi;
    //return (indirectBounceResult * senderArea) * oneOverPi;
}

void main() {
    vec2 uv = vertexOutput.uv;

    vec3 position = texture(gPosition, uv).rgb;
    vec3 normal = texture(gNormal, uv).rgb;
    vec3 albedo = texture(gAlbedo, uv).rgb;

    vec3 brdf = texture(diffuseBrdf, uv).rgb;

    vec3 viewSpacePosition = (camera.view * vec4(position, 1)).xyz;
    mat3 TBN = getCoordinateSystem(normal);
    TBN = getRotatedCoordinateSystem(normal, uv);

    float ssdoMipLevel = 1024;
    vec3 LiSSDO = directionalOcclusion(position, viewSpacePosition, TBN, normal, skybox, ssdoMipLevel);
    vec3 ssdoResult = LiSSDO * brdf;//(albedo);

	fragColor = vec4(ssdoResult, 1);
}

