#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;


uniform sampler2D gPositionHalfRes;
uniform sampler2D gNormalHalfRes;
uniform sampler2D brdfHalfRes;
uniform sampler2D ssdoScene;

out vec4 fragColor;

float area = 1;

float senderArea = (pi * (rMax * rMax)) / (NUM_SAMPLES * area);


vec3 indirectLighting(vec3 position, vec3 viewSpacePosition, mat3 TBN, vec3 normal) {
    vec3 result = vec3(0);

    for (int i = 0; i < NUM_SAMPLES; ++i) {
        vec3 sampleDirection = TBN * samples[i].xyz;
        vec3 samplePoint = position + sampleDirection * samples[i].w;

        vec4 viewSpaceSample = camera.view * vec4(samplePoint, 1);

        vec2 sampleUV = getSampleUV(viewSpaceSample);
        vec3 worldSpaceOccluder = texture(gPositionHalfRes, sampleUV).rgb;

        vec3 transmittanceDirection = worldSpaceOccluder - position;
        vec3 transmittanceDirectionNormalized = normalize(transmittanceDirection);

        float distance = min(1.0, length(transmittanceDirection));
        float distanceSquared = distance * distance;

        vec3 viewSpaceOccluder = (camera.view * vec4(worldSpaceOccluder, 1)).xyz;
        float visibility = 1.0 - getVisibility(viewSpaceSample.xyz, viewSpaceOccluder, distance);

        vec3 occluderNormal = texture(gNormalHalfRes, sampleUV).rgb;

        float cosThetaSender = max(0.0, dot(occluderNormal, -transmittanceDirectionNormalized));
        float cosThetaReceiver = max(0.0, dot(normal, transmittanceDirectionNormalized));

        vec3 li = texture(ssdoScene, sampleUV).rgb;

        result += max(vec3(0), li * visibility * ((cosThetaSender * cosThetaReceiver) / distanceSquared));
    }
    return result * oneOverPi * senderArea;
}

void main() {
    vec2 uv = vertexOutput.uv;
    vec3 color = texture(ssdoScene, uv).rgb;

    vec3 position = texture(gPositionHalfRes, uv).rgb;
    vec3 normal = texture(gNormalHalfRes, uv).rgb;
    vec3 diffuseBrdf = texture(brdfHalfRes, uv).rgb;

    vec3 viewSpacePosition = (camera.view * vec4(position, 1)).xyz;
    mat3 TBN = getCoordinateSystem(normal);
    TBN = getRotatedCoordinateSystem(normal, uv);

    vec3 diffuse = indirectLighting(position, viewSpacePosition, TBN, normal);

    diffuse = diffuse * diffuseBrdf;

    fragColor = vec4(diffuse + color, 1);
}

