#version 430

#include "global.inc"

in VertexOutput
{
	vec2 uv;
	vec2 surrounding[4];
} vertexOutput;

uniform sampler2D gPosition;
uniform sampler2D lightingResult;
uniform sampler2D downsampled;

uniform float volumetricIntensifier;

out vec4 fragColor;

int minDimension(vec4 v) {
    int dim = 0;
    float minComponent = v.x;
    for (int i = 1; i < 4; ++i) {
        if (v[i] < minComponent) {
            dim = i;
            minComponent = v[i];
        }
    }
    return dim;
}

void main() {
    vec2 fullResUV = vertexOutput.uv;

    vec4 fullResDepth = vec4((camera.view * vec4(texture(gPosition, fullResUV).xyz, 1.0)).z);
	vec4 lowResDepths = vec4(
		texture(downsampled, vertexOutput.surrounding[0]).z,
		texture(downsampled, vertexOutput.surrounding[1]).z,
		texture(downsampled, vertexOutput.surrounding[2]).z,
		texture(downsampled, vertexOutput.surrounding[3]).z
	);
	int dim = minDimension(abs(fullResDepth - lowResDepths));

    vec2 lowResUV = vertexOutput.surrounding[dim];
    vec4 highResColor = texture(lightingResult, fullResUV);
    vec4 lowResColor = texture(downsampled, lowResUV) * volumetricIntensifier;

    fragColor = fragColor = vec4(highResColor.xyz + lowResColor.xyz, 1);
}
