#version 430 core

#include "global.inc"

in VertexOutput
{
	vec2 uv;
} vertexOutput;

uniform sampler2D gPositionHalfRes;

out vec4 fragColor;

uniform float stepSize;
uniform vec3 mediumAlbedo;

const float PI_TIMES_FOUR = pi * 4;
const float INV_PI_TIMES_FOUR = 1 / PI_TIMES_FOUR;

struct Ray {
  vec3 origin;
  vec3 direction;
  vec3 intersection;
  float tMax;           // intersection = origin + direction * tMax;
  vec3 step;            // one step along the ray
};

Ray createRay(vec3 origin, vec3 intersection) {
    vec3 direction = intersection - origin;
    float tMax = length(direction);

    Ray charles;
    charles.origin = origin;
    charles.intersection = intersection;
    charles.direction = normalize(direction);
    charles.tMax = tMax;
    charles.step = charles.direction * stepSize;
    return charles;
}

vec3 directionalLightContribution(Ray charles, Light light) {
    vec3 contribution = vec3(0);

    vec3 L = normalize(-light.direction);
    float lightDistance = 1.0;
    vec3 intensity = light.color * light.intensity;

    mat4 viewProjection = lightViewProjection[light.shadowMapIndex];

    vec3 currentStep = charles.origin + charles.step;
    for (float t = stepSize; t < charles.tMax; t += stepSize) {
        float shadow = calcShadow(light, viewProjection * vec4(currentStep, 1));

        contribution += shadow * mediumAlbedo * intensity * INV_PI_TIMES_FOUR
            * exp(-lightDistance * mediumAlbedo) * exp(-t * mediumAlbedo) * stepSize;

        currentStep += charles.step;
    }

    return contribution;
}


void main() {
	vec2 uv = vertexOutput.uv;

	vec3 position = texture(gPositionHalfRes, uv).rgb;

	Ray charles = createRay(position, camera.position);
    vec3 LiVolumetric = vec3(0);
	for (int i = 0; i < NUM_LIGHTS; ++i) {
	    LiVolumetric += directionalLightContribution(charles, lights[i]);
	}

    vec3 viewSpacePosition = (camera.view * vec4(position, 1.0)).xyz;

	fragColor = vec4(LiVolumetric, viewSpacePosition.z);
}

